local myname, mm = ...
local _G = getfenv(0)

local select	= _G.select

local err = mm.err
local L = mm.L
local version = _G.GetAddOnMetadata(myname, "Version")


local myOptionsTable = {
	name = ("%s - %s"):format(myname,version),
	type = "group",
	order = 1,
	icon = "Interface\\Icons\\INV_Misc_Book_01",
	args = {
		header1 = {
			type = 'header',
			name = "",
			width = "full",
			order = 0.2,
		},
		enable = {
			name = (L["Enable %s"]):format(myname),
			desc = L["Enables / disables the addon"],
			type = "toggle",
			width = "full",
			set = function(info,val) 
				mm.sessionDB.enabled = val 
				if val then
					mm:Enable()
				else
					mm:Disable()
				end
			end,
			get = function(info) return mm.sessionDB.enabled end,
			order = 0.1,
		},
		weight = {
			name = L["Random Weight for Non Favorite"],
			desc = L["Weight for the non-favorite mounts from 0 to 100. Use 0 if you only want the favorite mounts to be selected. Favorite mounts have a weight of 100."],
			type = "range",
			width = "double",
			min = 0,
			max = 100,
			step = 5,
			get = function() return mm.db.weight_for_nonfevarite*100 end,
			set = function(info,val) 
				mm.db.weight_for_nonfevarite = val/100 
				mm:RefreshMountList()
			end,
			order = 0.3,
		},
		weight2 = {
			name = L["Random Weight for Class Mounts"],
			desc = L["Weight for the class mounts like the Warlock's Dreadsteed from 0 to 100. Use 0 if you do not want these mounts to be selected."],
			type = "range",
			width = "double",
			min = 0,
			max = 100,
			step = 5,
			get = function() return mm.db.weight_for_class_mounts*100 end,
			set = function(info,val) 
				mm.db.weight_for_class_mounts = val/100
				mm:RefreshMountList()
			end,
			order = 0.35,
		},
		macro1 = {
			name = L["Macro Template"],
			desc = L["Template used to create the caracter specific mm macro. Use /macro to drag on one of your action buttons."],
			descStyle = "inline",
			type = "input",
			width = "full",
			multiline = 6,
			get = function() return mm.db.mm1 end,
			set = function(info,val) 
				mm.db.mm1 = val
				mm:RefreshMacro()
				_G.LibStub("AceConfigRegistry-3.0"):NotifyChange(myname)
			end,
			order = 0.4,
		},
		tokens = {
			name = [[
MacroMount creates a character specific macro named mm. Drag it to one of your action buttons to use.

<g> = Ground Mount
<f> = Flying Mount
<fg> = Ground or Flying Mount
<r> = Repair Mount
<uw> = Under Water Mount (Swim Bonus)
<ww> = Water Walking Mount
<pf> = Flying Passenger Mount
<pg> = Ground Passenger Mount
<pfg> = Flying or Ground Passenger Mount
<cm> = Current Mount (last one that was summoned)
]],
			type = "description",
			order = 0.45,
		},
		reset_template1 = {
			name = L["Reset Macro Template"],
			desc = L["Reset the macro to the default value."],
			type = "execute",
			func = function()
				mm.db.mm1 = mm.sessionDB.macro_template[select(2,_G.UnitClass("player"))] or mm.sessionDB.macro_template.default
				mm:RefreshMacro()
				_G.LibStub("AceConfigRegistry-3.0"):NotifyChange(myname)
			end,
			order = 0.5,
		},
		refresh_macro1 = {
			name = L["Refresh Macro"],
			desc = L["Regenerate the macro based on the current template."],
			type = "execute",
			func = function()
				mm:RefreshMacro()
				_G.LibStub("AceConfigRegistry-3.0"):NotifyChange(myname)
			end,
			order = 0.6,
		},
		preview_macro1 = {
			name = L["Resulting macro"],
			type = "input",
			width = "full",
			multiline = 6,
			get = function() 
				return _G.GetMacroBody(mm.sessionDB.macroIndex)
			end,
			order = 0.7,
		},
	}
}

function mm:SetupOptions()
	-- Profile section
	--options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(Syllabus.db)

	-- Init the Blizzard options pannel
	_G.LibStub("AceConfig-3.0"):RegisterOptionsTable(myname, myOptionsTable)
	_G.LibStub("AceConfigDialog-3.0"):AddToBlizOptions(myname)

	-- Profile management
	local p_options = _G.LibStub("AceDBOptions-3.0"):GetOptionsTable(self.acedb)
	_G.LibStub("AceConfig-3.0"):RegisterOptionsTable(myname..'-Profile', p_options)
	_G.LibStub("AceConfigDialog-3.0"):AddToBlizOptions(myname..'-Profile', p_options.name, myname)

	-- Profile callbacks
	self.acedb.RegisterCallback(self, "OnProfileChanged", "RefreshMacro")
	self.acedb.RegisterCallback(self, "OnProfileCopied", "RefreshMacro")
	self.acedb.RegisterCallback(self, "OnProfileReset", "RefreshMacro")


	-- About section (only if someone else has included LibAboutPanel)
	if _G.LibStub:GetLibrary("LibAboutPanel", true) then
		_G.LibStub:GetLibrary("LibAboutPanel").new(myname, myname)
	end
end