
local myname, mm = ...
local _G = getfenv(0)

local pairs = _G.pairs
local select = _G.select
local tinsert = _G.tinsert
local wipe = _G.wipe

local err = mm.err

mm.defaults = {
	profile = {
		weight_for_nonfevarite = .05,
		weight_for_class_mounts = 1
	},
}
mm.defaultsPC = {
	weight_for_nonfevarite = .05,
}
mm.defaultsSession = {
	enabled = true,
	-- All three tables will always be in sync i.e. position ml_id.g[n], ml_name.g[n] and ml_weight[n] are for the same mount
	-- Table of mount spellID by type
	ml_id = {
		g = {}, 
		f = {}, 
		fg = {},
		ww = {}, 
		uw = {}, 
		tfg = {}, 
		tff = {}, 
		r = {},
		vashjir = {},
		ahnqiraj = {},
	},
	-- Table of mount name by type
	ml_name = {
		g = {}, 
		f = {}, 
		fg = {},
		ww = {}, 
		uw = {}, 
		tfg = {}, 
		tff = {}, 
		r = {},
		vashjir = {},
		ahnqiraj = {},
	},
	-- Table of isFavorite value by type
	ml_weight = {
		g = {}, 
		f = {}, 
		fg = {},
		ww = {}, 
		uw = {}, 
		tfg = {}, 
		tff = {}, 
		r = {},
		vashjir = {},
		ahnqiraj = {},
	},
	is_a_mount = {}
}


function mm:InitDB()
	--_G[myname.."DB"] = _G.setmetatable(_G[myname.."DB"] or {}, {__index = mm.defaults})
	--mm.sessionDB = _G[myname.."DB"]
	--_G[myname.."DB"] = {}
	-- Init AceDB 
	self.acedb = _G.LibStub("AceDB-3.0"):New(myname.."DB", self.defaults, true)
	self.db = self.acedb.profile

	--_G[myname.."DBPC"] = _G.setmetatable(_G[myname.."DBPC"] or {}, {__index = mm.defaultsPC})
	--mm.dbpc = _G[myname.."DBPC"]
	-- Init character specific saved variables
	self.dbpc = self.db.char

	-- Init sessions variables, will be discarded when the player logs out
	self.sessionDB = self.defaultsSession
end


function mm:FlushDB()
	--for i,v in pairs(mm.defaults) do if mm.sessionDB[i] == v then mm.sessionDB[i] = nil end end
	--for i,v in pairs(mm.defaultsPC) do if mm.dbpc[i] == v then mm.dbpc[i] = nil end end
end

--[==[	Pre-defined mount lists	]==]
-- Information from http://www.warcraftmounts.com/ and http"//www.wowhead.com

local travel_with_friend = {
	[55531] = true, 	-- Mechano-Hog
	[60425] = true, 	-- Mekgineer's Chopper
	[61425] = true, 	-- Traveler's Tundra Mammoth (Alliance)
	[61447] = true, 	-- Traveler's Tundra Mammoth (Horde)
	[61465] = true, 	-- Grand Black War Mammoth (Alliance)
	[61467] = true, 	-- Grand Black War Mammoth (Horde)
	[61469] = true, 	-- Grand Ice Mammoth (Horde)
	[61470] = true, 	-- Grand Ice Mammoth (Alliance)
	[75973] = true, 	-- X-53 Touring Rocket
	[93326] = true, 	-- Sandstone Drake
	[121820] = true, 	-- Obsidian Nightwing
	[122708] = true, 	-- Grand Expedition Yak
}

local repair = {
	[61425] = true, 	-- Traveler's Tundra Mammoth (Alliance)
	[61447] = true, 	-- Traveler's Tundra Mammoth (Horde)
	[122708] = true, 	-- Grand Expedition Yak
}

-- Summonable mounts that are not listed in the mount journal
local unlisted_gound_mount_spell = {
-- spellID   isFavorite
	[23214] = true,	-- Summon Charger
	[73630] = true,	-- Summon Great Exarch's Elekk
	[69826] = true,	-- Summon Great Sunwalker Kodo
	[34767] = true,	-- Summon Thalassian Charger
	[73629] = true,	-- Summon Exarch's Elekk
	[69820] = true,	-- Summon Sunwalker Kodo
	[34769] = true,	-- Summon Thalassian Warhorse
	[13819] = true,	-- Summon Warhorse
	[23161] = true,	-- Dreadsteed
	[5784] = true,		-- Felsteed
	[48778] = true,	-- Acherus Deathcharger
}

--[==[
	
	Gather the lists of mounts
	--------------------------

	g 			= Ground
	f 			= Flying
	fg			= Ground + Flying that can also be used on the ground
	ww 		= Water Walking
	uw 		= Under Water
	tff 		= Transport Friend while Flying
	tfg		= Transport Friend on the ground
	r 			= Repair
	vashjir	= Can only be used in Vashj'Ir while swimming
	ahnqiraj	= Can only be used in the Temple of Ahn'Qiraj
	
  ]==]

function mm:RefreshMountList()
	local ml_id, ml_name, ml_weight, is_a_mount = self.sessionDB.ml_id, self.sessionDB.ml_name, self.sessionDB.ml_weight, self.sessionDB.is_a_mount
	local faction_name = _G.UnitFactionGroup("player")
	local faction_id =  (faction_name == "Horde" and 0) or (faction_name == "Alliance" and 1) or 2

	self.sessionDB.mountbytype = {}
	for k,v in pairs(ml_id) do wipe(v) end
	for k,v in pairs(ml_name) do wipe(v) end
	for k,v in pairs(ml_weight) do wipe(v) end

	for i=1, _G.C_MountJournal.GetNumMounts() do
		local creatureName, spellID, icon, active, isUsable, sourceType, isFavorite, isFactionSpecific, faction, hideOnChar, isCollected = _G.C_MountJournal.GetMountInfo(i)
		local creatureDisplayID, descriptionText, sourceText, isSelfMount, mountType = _G.C_MountJournal.GetMountInfoExtra(i)
		local isFavorite = _G.C_MountJournal.GetIsFavorite(i)

		-- Use the spell name instead of the creature name since it sometimes differ
		if spellID then
			creatureName = _G.GetSpellInfo(spellID)
		end

		if isCollected and not hideOnChar and (not isFactionSpecific or faction == faction_id) then
			-- Remember all the valid mounts
			is_a_mount[spellID] = true

			-- The weight is 1 for favorite mount, weight_for_class_mounts for class mount and
			-- weight_for_nonfevarite for all the other mounts
			local weight = (isFavorite and 1) or 
								(unlisted_gound_mount_spell[spellID] and self.db.weight_for_class_mounts) or
								self.db.weight_for_nonfevarite

			if 		mountType == 230 then
				-- Ground mount
				tinsert(ml_id.g, spellID)
				tinsert(ml_name.g, creatureName)
				tinsert(ml_weight.g, weight)
				tinsert(ml_id.fg, spellID)
				tinsert(ml_name.fg, creatureName)
				tinsert(ml_weight.fg, weight)
			elseif 	mountType == 231 then
				-- Swim speed bonus mount
				tinsert(ml_id.uw, spellID)
				tinsert(ml_name.uw, creatureName)
				tinsert(ml_weight.uw, weight)
			elseif 	mountType == 254 then
				-- Swim speed bonus but not usable on land
				tinsert(ml_id.uw, spellID)
				tinsert(ml_name.uw, creatureName)
				tinsert(ml_weight.uw, weight)
				tinsert(ml_id.vashjir, spellID)
				tinsert(ml_name.vashjir, creatureName)
				tinsert(ml_weight.vashjir, weight)
			elseif 	mountType == 247 then
				-- Flying mount, only contains Red Flying Cloud for now, don't know why it has its own mountType
				tinsert(ml_id.f, spellID)
				tinsert(ml_name.f, creatureName)
				tinsert(ml_weight.f, weight)
				tinsert(ml_id.fg, spellID)
				tinsert(ml_name.fg, creatureName)
				tinsert(ml_weight.fg, weight)
			elseif 	mountType == 269 then
				-- Water walking mount
				tinsert(ml_id.ww, spellID)
				tinsert(ml_name.ww, creatureName)
				tinsert(ml_weight.ww, weight)
			elseif 	mountType == 248 then
				-- Flying mount that can also be used a ground mount
				tinsert(ml_id.f, spellID)
				tinsert(ml_name.f, creatureName)
				tinsert(ml_weight.f, weight)
				tinsert(ml_id.fg, spellID)
				tinsert(ml_name.fg, creatureName)
				tinsert(ml_weight.fg, weight)
			elseif 	mountType == 241 then
				-- Ground mount that can only be used in the Temple of Ahn'Qiraj
				-- (The API is lying, the red one is faster)
				tinsert(ml_id.ahnqiraj, spellID)
				tinsert(ml_name.ahnqiraj, creatureName)
				tinsert(ml_weight.ahnqiraj, weight)
			elseif 	mountType == 232 then
				-- Swiming mount that can only be used in Vashj'ir
				tinsert(ml_id.vashjir, spellID)
				tinsert(ml_name.vashjir, creatureName)
				tinsert(ml_weight.vashjir, weight)
			else
				err("Unknown mountType = %s, name = %s, spellID = %s, id = %s", mountType, creatureName, spellID, i)
			end

			-- Special cases that are not specified by the API
			if travel_with_friend[spellID] then
				if mountType == 248 then
					tinsert(ml_id.tff, spellID)
					tinsert(ml_name.tff, creatureName)
					tinsert(ml_weight.tff, weight)
				elseif mountType == 230 then
					tinsert(ml_id.tfg, spellID)
					tinsert(ml_name.tfg, creatureName)
					tinsert(ml_weight.tfg, weight)
				else
					err("Unknown mountType = %s, name = %s, spellID = %s, id = %s", mountType, creatureName, spellID, i)
				end
			end

			if repair[spellID] then
				tinsert(ml_id.r, spellID)
				tinsert(ml_name.r, creatureName)
				tinsert(ml_weight.r, weight)
			end
		end

		-- For debug
		self.sessionDB.mountbytype[mountType] = self.sessionDB.mountbytype[mountType] or {}
		self.sessionDB.mountbytype[mountType][("%s : %s : isSelfMount = %s : isUsable = %s : hideOnChar = %s"):format(_G.tostringall(creatureName, spellID, isSelfMount, isUsable, hideOnChar))] = true
	end

	-- Special case for Warlocks with the Glyph of Nightmares
	if select(2, _G.UnitClass("player")) == "WARLOCK" and (is_a_mount[23161] or is_a_mount[5784]) then
		for i=1,_G.GetNumGlyphSockets() do
			if select(4,_G.GetGlyphSocketInfo(i)) == 56232 then
				if is_a_mount[23161] then	
					-- Dreadsteed			
					local name =  _G.GetSpellInfo(23161)
					tinsert(ml_id.ww, 23161)
					tinsert(ml_name.ww, name)
					tinsert(ml_weight.ww, self.db.weight_for_class_mounts)
				end
				if is_a_mount[5784] then			
					-- Felsteed	
					local name =  _G.GetSpellInfo(5784)
					tinsert(ml_id.ww, 5784)
					tinsert(ml_name.ww, name)
					tinsert(ml_weight.ww, self.db.weight_for_class_mounts)
				end
			end
		end
	end
end
